-------------------------------------------------------------------------------
-- 8-bit x 8-bit pipelined multiplier
--
-- Ports:
--   - clk [in]  : clock signal
--   - x   [in]  : operand 1
--   - y   [in]  : operand 2
--   - o   [out] : result = x * y
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity mult is
  port ( clk : in  std_logic;
         x   : in  std_logic_vector(7 downto 0);
         y   : in  std_logic_vector(7 downto 0);
         o   : out std_logic_vector(15 downto 0) );
end entity;

architecture arch of mult is
  signal pp_0 : std_logic_vector(8*8-1 downto 0);
  signal pp_1 : std_logic_vector(8*8-1 downto 0);

  signal sum1_1 : std_logic_vector(4*10-1 downto 0);
  signal sum1_2 : std_logic_vector(4*10-1 downto 0);

  signal sum2_2 : std_logic_vector(2*12-1 downto 0);
  signal sum2_3 : std_logic_vector(2*12-1 downto 0);

  signal sum3_3 : std_logic_vector(15 downto 0);
  signal sum3_4 : std_logic_vector(15 downto 0);
begin

  -- Partial products
  pp_y : for j in 7 downto 0 generate
    pp_x : for i in 7 downto 0 generate
      pp_0(8*j+i) <= x(i) and y(j);
    end generate;
  end generate;

  -- Registers for the PPs
  process(clk)
  begin
    if clk'event and clk = '1' then
      pp_1 <= pp_0;
    end if;
  end process;

  -- First stage of additions
  sum1_1( 9 downto  0) <= ("00" & pp_1( 7 downto  0)) + ("0" & pp_1(15 downto  8) & "0");
  sum1_1(19 downto 10) <= ("00" & pp_1(23 downto 16)) + ("0" & pp_1(31 downto 24) & "0");
  sum1_1(29 downto 20) <= ("00" & pp_1(39 downto 32)) + ("0" & pp_1(47 downto 40) & "0");
  sum1_1(39 downto 30) <= ("00" & pp_1(55 downto 48)) + ("0" & pp_1(63 downto 56) & "0");

  -- Registers for the first sums
  process(clk)
  begin
    if clk'event and clk = '1' then
      sum1_2 <= sum1_1;
    end if;
  end process;

  -- Second stage of additions
  sum2_2(11 downto  0) <= ("00" & sum1_2( 9 downto  0)) + ("00" & sum1_2(19 downto 10));
  sum2_2(23 downto 12) <= ("00" & sum1_2(29 downto 20)) + (sum1_2(39 downto 30) & "00");

  -- Registers for the second sums
  process(clk)
  begin
    if clk'event and clk = '1' then
      sum2_3 <= sum2_2;
    end if;
  end process;

  -- Last addition
  sum3_3 <= ("0000" & sum2_3(11 downto 0)) + (sum2_3(23 downto 12) & "0000");

  -- Registers for last sum
  process(clk)
  begin
    if clk'event and clk = '1' then
      sum3_4 <= sum3_3;
    end if;
  end process;

  o <= sum3_4;

end architecture;

