library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;

entity mult_test is
end entity;

architecture arch of mult_test is
  signal done   : boolean := false;
  signal passed : boolean := true;
  signal ok     : boolean := true;

  component mult is
    port ( clk : in  std_logic;
           x   : in  std_logic_vector(7 downto 0);
           y   : in  std_logic_vector(7 downto 0);
           o   : out std_logic_vector(15 downto 0) );
  end component;

  signal clk : std_logic;
  signal x   : std_logic_vector(7 downto 0);
  signal y   : std_logic_vector(7 downto 0);
  signal o   : std_logic_vector(15 downto 0);
begin

  mult_0 : mult
    port map ( clk => clk,
               x   => x,
               y   => y,
               o   => o );

  -----------------------------------------------------------------------------
  -- Clock signal
  -----------------------------------------------------------------------------

  process
  begin
    if not done then
      clk <= '1';
      wait for 5 ns;
      clk <= '0';
      wait for 5 ns;
    else
      clk <= 'U';
      wait;
    end if;
  end process;

  -----------------------------------------------------------------------------
  -- Input stimuli
  -----------------------------------------------------------------------------

  process
  begin
    ---------------------------------------------------------------------------
    for i in 0 to 255 loop
      for j in 0 to 255 loop
        x <= conv_std_logic_vector(i, 8);
        y <= conv_std_logic_vector(j, 8);
        wait for 10 ns;
      end loop;
    end loop;
    ---------------------------------------------------------------------------
    x <= "UUUUUUUU";
    y <= "UUUUUUUU";
    wait;
  end process;
  
  -----------------------------------------------------------------------------
  -- Output verification
  -----------------------------------------------------------------------------

  process
  begin
    wait for 41 ns;
    ---------------------------------------------------------------------------
    for i in 0 to 255 loop
      for j in 0 to 255 loop
        ok <= o = conv_std_logic_vector(i*j, 16);
        wait for 10 ns;
      end loop;
    end loop;
    ---------------------------------------------------------------------------
    ok   <= true;
    done <= true;
    wait;
  end process;

  passed <= passed and ok;

end architecture;

